;*********************************************************************
.model tiny
.486p
.code

C_ESC EQU 1BH
C_CR  EQU 0DH
C_LF  EQU 0AH
C_SOH EQU 01H
C_EOT EQU 04H
C_ACK EQU 06H
C_NAK EQU 15H

	org 0
start:
	jmp nustart

;*********************************************************************
; Data Area
PKT_SIZE		EQU 67
dbuf			db PKT_SIZE dup (0)
dbuf_idx		db 0
val_pkt_idx db 1
err_cnt		db 0
dummy  db "dummy"
commmsg db "Serial Port Flash Burner: ", 0

;*********************************************************************
; Macro to read 8 bit ZF logic register to al
READ_ZFL08   macro reg
				mov al, reg
				mov dx, ZFL_CTL_PORT
				out dx, al
				mov dx, ZFL_DATB_PORT
				in  al, dx
				endm

;*********************************************************************
; Macro to write 32 bit port in ZF logic registers
WRITE_ZFL08 macro reg, data
				mov  al, data
				push ax
				mov dx, ZFL_CTL_PORT
				mov al, reg
				out dx, al
				mov dx, ZFL_DATB_PORT
				pop ax
				out dx, al
				endm

;*********************************************************************
; Macro to write 32 bit port in ZF logic registers
WRITE_ZFL32 macro reg, data
				mov eax, data
				push eax
				mov dx, ZFL_CTL_PORT
				mov al, reg
				out dx, al
				mov dx, ZFL_DATW_PORT
				pop eax
				out dx, eax
				endm


;*********************************************************************
; Bring in the BURAPI
	include burapi.asm
; And the flash burner
;	include intfl64.asm
	include atmflsh.asm
;	include intfl256.asm

;*********************************************************************
; Receive a packet from the serial port using our simple minded protocol.
; Basic protocol design: Receiver sends an ACK. Sender sends an STX
; followed by 16 bytes and an 8 bit checksum (data only). Receiver
; sends either an ACK or a NAK to indicate the packet was received
; correctly. If NAKed, receiver flushes the packet and sender
; resends. Receiver can abort process at any time by sending an ESC.
; Sender will send an ESC indicating the process is complete.
;
; return value in AL: 0=Pkt received and valid
;							 1=ESC received
;							 2=EOT received
;							 3=timeout error
;							 4=Pkt received and invalid
;							 5=Invalid head byte

RcvPkt	proc near
			push bx
			push cx
			push si
			push di
			push ds
			push cs
			pop ds
			xor ax,ax
			mov dbuf_idx, al
			mov di, offset dbuf
rp_pktloop:
			mov cx, 5
rp_getch:
			; Wait for a single character up to 220 msec.
			call SerRecWait
			jnz rp_got_one
			dec cx
			jnz rp_getch
			; tried 5 times to no avail. return timeout error
			mov ax, 3
			jmp short rp_quit

rp_got_one:
			mov ds:[di], al
			inc di
			inc ds:dbuf_idx
			; If the first char in pkt
			cmp ds:dbuf_idx, 1
			jne rp_fini_loop
			; First char. If an escape, we be done
			cmp al, C_ESC
			jne rp_1
			mov ax, 1
			jmp short rp_quit
rp_1:
			cmp al, C_EOT
			jne rp_2
			mov ax, 2
			jmp short rp_quit
rp_2:
			cmp al, C_SOH
			je rp_fini_loop
			mov ax, 5
			jmp short rp_quit

rp_fini_loop:
			cmp ds:dbuf_idx, PKT_SIZE
			jl rp_pktloop

rp_confirm:
			; Now have a complete pkt. Confirm checksum
			xor bx,bx
			xor ax,ax
rp_csum:
			add ah, ds:dbuf[bx+2]
			inc bx
			cmp bx, PKT_SIZE-3
			jne rp_csum

			; Compare csum
			mov bx, ax
			xor ax, ax
			cmp bh, ds:dbuf[PKT_SIZE-1]
			je rp_quit
			mov ax, 4
rp_quit:
			push ax
			pop bx
			mov ax, C_ACK
			cmp bx, 3
			jle rp_sndresp
			mov ax, C_NAK
			call SerSend2
rp_sndresp:
         cmp bx, 2               ; Send an ACK only for an EOT.  For data
         jnz rp_skip             ;  packets, the ACK is sent after the call to
 			call SerSend2           ;  flash_burn_pkt.
rp_skip:
			mov ax, bx
			pop ds
			pop di
			pop si
			pop cx
			pop bx
			ret
RcvPkt	endp
;*********************************************************************
set_bus_speed proc
			; Set ISA bus speed - reused from MachZ databook V.758, p455
			pushf
			cli
			mov eax, 80009050H
			mov dx, 0cf8H
			out dx, eax
			mov dx, 0cfcH
			in al, dx
			or al, 00000111b
			out dx, al
			popf
			ret
set_bus_speed endp
;*********************************************************************
; Start of code
nustart:
			; Send the opening msg
			push cs
			pop es

			; Set the bus speed for time driven events
			call set_bus_speed

			; Tell user what we be doing
			mov di, offset commmsg
			xor cx,cx
			call SerSend
			call flash_banner
			call CRLF

			; Confirm flash programmer/flash compatibility
			call flash_confirm
			cmp al, 0
			jne ns_quit

ns_good_flash:
			; Flash is compatible. Erase it.
			call flash_erase_segment
			cmp al, 0
			jne ns_quit

			; Tell sender we be ready
			mov al, C_ACK
			call SerSend2

ns_rcv_loop:
			; Loop to receive a pkt and burn it
			call RcvPkt
			cmp al, 0
			jne ns_pr_err
			mov bh, cs:val_pkt_idx
			cmp cs:dbuf[1], bh
			jne ns_wrong_pkt
			inc cs:val_pkt_idx
			mov cs:err_cnt, al
			call flash_burn_pkt
			cmp al, 0
         jnz ns_quit
         mov   ax, C_ACK                        ; Send the ACK to the client
         call  SerSend2
			jmp ns_rcv_loop

wrong_pkt_msg db C_ESC, "Pkt serial number out of sequence", C_CR, C_LF, 0
ns_wrong_pkt:
			xor cx, cx
			mov di, offset wrong_pkt_msg
			call SerSend
			jmp ns_rcv_loop

ns_pr_err:
			; Rcv_Pkt returned something other than a valid pkt.
			inc cs:[err_cnt]
			cmp cs:[err_cnt], 5
			jg ns_errcnt
			; Check for EOT
			cmp al, 2
			je ns_fini
			; Check for ESC (abort transfer)
			cmp al, 1
			je ns_abort
			; all other errors are protocol issues. Allow error loop to
			; address
			jmp ns_rcv_loop

errcnt_msg db C_ESC, "Pkt error count exceeded", C_CR, C_LF, 0
ns_errcnt:
			xor cx, cx
			mov di, offset errcnt_msg
			call SerSend
			jmp ns_quit

abort_msg db C_ESC, "ESC Received: Aborting.", C_CR, C_LF, 0
ns_abort:
			xor cx, cx
			mov di, offset abort_msg
			call SerSend
			jmp ns_quit

alldone_msg db C_ESC, C_CR, C_LF
				db "Serial flash programming complete. Exiting to monitor.", C_CR, C_LF, 0
ns_fini:
			xor cx, cx
			mov di, offset alldone_msg
			call SerSend

ns_quit:
			; flush the receive queue for any pending data.
			call SerRecWait
			jnz ns_quit
			retf

end start


ends

